@@ -8,20 +8,7 @@
 #ifndef SSILINK_H
 #define SSILINK_H
 
-#include <Singular/links/silink.h>
-
-typedef struct
-{
-  s_buff f_read;
-  FILE *f_write;
-  ring r;
-  pid_t pid; /* only valid for fork/tcp mode*/
-  int fd_read,fd_write; /* only valid for fork/tcp mode*/
-  char level;
-  char send_quit_at_exit;
-  char quit_sent;
-
-} ssiInfo;
+#include "Singular/links/silink.h"
 
 BOOLEAN ssiOpen(si_link l, short flag, leftv u);
 BOOLEAN ssiWrite(si_link l, leftv v);
@@ -34,4 +21,7 @@ si_link_extension slInitSsiExtension(si_link_extension s);
 si_link ssiCommandLink();
 
 void sig_chld_hdl(int sig);
+int ssiReservePort(int clients);
+
+void singular_close_links();
 #endif
