@@ -27,11 +27,15 @@
 #define HAVE_TAIL_RING
 
 #define setmax 128
-#define setmaxL ((4096-12)/sizeof(LObject))
-#define setmaxLinc ((4096)/sizeof(LObject))
+#define setmaxL ((int)((4096-12)/sizeof(LObject)))
+#define setmaxLinc ((int)((4096)/sizeof(LObject)))
 
-#define setmaxT ((4096-12)/sizeof(TObject))
-#define setmaxTinc ((4096)/sizeof(TObject))
+#define setmaxT ((int)((4096-12)/sizeof(TObject)))
+#define setmaxTinc ((int)((4096)/sizeof(TObject)))
+
+#define RED_CANONICALIZE 200
+#define REDNF_CANONICALIZE 60
+#define REDTAIL_CANONICALIZE 100
 
 // if you want std computations as in Singular version < 2:
 // This disables RedThrough, tailReductions against T (bba),
@@ -262,7 +266,6 @@ public:
   KINLINE void HeadNormalize();
 };
 
-
 EXTERN_VAR int HCord;
 
 class skStrategy
@@ -323,11 +326,9 @@ public:
   TSet T;
   LSet L;
   LSet    B;
-  poly    kHEdge;
   poly    kNoether;
-  poly    t_kHEdge; // same polys in tailring
+  poly    t_kNoether; // same polys in tailring
   KINLINE poly    kNoetherTail();
-  poly    t_kNoether;
   BOOLEAN * NotUsedAxis;
   BOOLEAN * pairtest;/*used for enterOnePair*/
   poly tail;
@@ -351,11 +352,9 @@ public:
   int Bl,Bmax;
   int ak,LazyDegree,LazyPass;
   int syzComp;
-  int HCord;
   int lastAxis;
   int newIdeal;
   int minim;
-  #ifdef HAVE_RINGS
   bool sigdrop; //This is used to check sigdrop in sba over Z
   int nrsyzcrit; // counts how many pairs are deleted by SyzCrit
   int nrrewcrit; // counts how many pairs are deleted by FaugereRewCrit
@@ -363,7 +362,6 @@ public:
                  // add directly sbaEnterS elements into S
   int blockred;  // counter for blocked reductions in redSig
   int blockredmax;
-  #endif
   #ifdef HAVE_SHIFTBBA
   int cv; // in shift bases: counting V criterion
   /*BOOLEAN*/ char rightGB;
@@ -373,7 +371,7 @@ public:
 #ifdef HAVE_PLURAL
   /*BOOLEAN*/ char z2homog; // Z_2 - homogeneous input allows product criterion in commutative and SCA cases!
 #endif
-  /*BOOLEAN*/ char kHEdgeFound;
+  /*BOOLEAN*/ char kAllAxis; // all axis are used -> (re)compute noether
   /*BOOLEAN*/ char honey,sugarCrit;
   /*BOOLEAN*/ char Gebauer,noTailReduction;
   /*BOOLEAN*/ char fromT;
@@ -416,7 +414,6 @@ public:
 void deleteHC(poly *p, int *e, int *l, kStrategy strat);
 void deleteHC(LObject* L, kStrategy strat, BOOLEAN fromNext = FALSE);
 void deleteInS (int i,kStrategy strat);
-void deleteInSSba (int i,kStrategy strat);
 void cleanT (kStrategy strat);
 static inline LSet initL (int nr=setmaxL)
 { return (LSet)omAlloc(nr*sizeof(LObject)); }
@@ -461,8 +458,6 @@ int posInLSig (const LSet set, const int length,
                LObject* L,const kStrategy strat);
 int posInLSigRing (const LSet set, const int length,
                LObject* L,const kStrategy strat);
-int posInLRing (const LSet set, const int length,
-               LObject* L,const kStrategy strat);
 int posInSyz (const kStrategy strat, const poly sig);
 int posInL0 (const LSet set, const int length,
              LObject* L,const kStrategy strat);
@@ -490,11 +485,12 @@ int posInL110 (const LSet set, const int length,
              LObject* L,const kStrategy strat);
 KINLINE poly redtailBba (poly p,int end_pos,kStrategy strat,BOOLEAN normalize=FALSE);
 KINLINE poly redtailBbaBound (poly p,int end_pos,kStrategy strat,int bound,BOOLEAN normalize=FALSE);
-#ifdef HAVE_RINGS
+KINLINE poly redtailBba_Ring (poly p,int end_pos,kStrategy strat);
 KINLINE poly redtailBba_Z (poly p,int end_pos,kStrategy strat);
+poly redtailBba_NF (poly p, kStrategy strat );
+poly redtailBba_Ring (LObject* L, int end_pos, kStrategy strat );
 poly redtailBba_Z (LObject* L, int end_pos, kStrategy strat );
 void redtailBbaAlsoLC_Z (LObject* L, int end_pos, kStrategy strat );
-#endif
 poly redtailBba (LObject *L, int end_pos,kStrategy strat,
                  BOOLEAN withT = FALSE,BOOLEAN normalize=FALSE);
 poly redtailBbaBound (LObject *L, int end_pos,kStrategy strat,int bound,
@@ -508,8 +504,8 @@ poly redNF (poly h,int & max_ind,int nonorm,kStrategy strat);
 int redNF0 (LObject *P,kStrategy strat);
 poly redNFTail (poly h,const int sl,kStrategy strat);
 int redHoney (LObject* h, kStrategy strat);
+int redHoneyM (LObject* h, kStrategy strat);
 int redLiftstd (LObject* h, kStrategy strat);
-#ifdef HAVE_RINGS
 int redRing (LObject* h,kStrategy strat);
 int redRing_Z (LObject* h,kStrategy strat);
 int redRiloc (LObject* h,kStrategy strat);
@@ -517,13 +513,6 @@ void enterExtendedSpoly(poly h,kStrategy strat);
 void enterExtendedSpolySig(poly h,poly hSig,kStrategy strat);
 void superenterpairs (poly h,int k,int ecart,int pos,kStrategy strat, int atR = -1);
 void superenterpairsSig (poly h,poly hSig,int hFrom,int k,int ecart,int pos,kStrategy strat, int atR = -1);
-poly kCreateZeroPoly(long exp[], long cabsind, poly* t_p, ring leadRing, ring tailRing);
-long ind2(long arg);
-
-long ind_fact_2(long arg);
-long twoPow(long arg);
-ideal createG0();
-#endif
 int redLazy (LObject* h,kStrategy strat);
 int redHomog (LObject* h,kStrategy strat);
 int redSig (LObject* h,kStrategy strat);
@@ -562,9 +551,7 @@ void updateS(BOOLEAN toT,kStrategy strat);
 void enterSyz (LObject &p,kStrategy strat, int atT);
 void enterT (LObject &p,kStrategy strat, int atT = -1);
 void replaceInLAndSAndT(LObject &p, int tj, kStrategy strat);
-#ifdef HAVE_RINGS
 void enterT_strong (LObject &p,kStrategy strat, int atT = -1);
-#endif
 void cancelunit (LObject* p,BOOLEAN inNF=FALSE);
 void HEckeTest (poly pp,kStrategy strat);
 void initBuchMoraCrit(kStrategy strat);
@@ -603,6 +590,7 @@ int kFindInTShift(poly p, TSet T, int tlength);
 /// return -1 if no divisor is found
 ///        number of first divisor in T, otherwise
 int kFindDivisibleByInT(const kStrategy strat, const LObject* L, const int start=0);
+int kFindDivisibleByInT_ecart(const kStrategy strat, const LObject* L, const int ecart);
 int kFindDivisibleByInT_Z(const kStrategy strat, const LObject* L, const int start=0);
 int kFindSameLMInT_Z(const kStrategy strat, const LObject* L, const int start=0);
 
@@ -640,27 +628,25 @@ KINLINE poly k_LmShallowCopyDelete_tailRing_2_currRing(poly p, ring tailRing);
 // return FALSE and m1 == NULL, m2 == NULL     , otherwise
 KINLINE BOOLEAN k_GetLeadTerms(const poly p1, const poly p2, const ring p_r,
                                poly &m1, poly &m2, const ring m_r);
-#ifdef HAVE_RINGS
 KINLINE void k_GetStrongLeadTerms(const poly p1, const poly p2, const ring leadRing,
                                poly &m1, poly &m2, poly &lcm, const ring taiRing);
-#endif
 #ifdef KDEBUG
 // test strat
 BOOLEAN kTest(kStrategy strat);
 // test strat, and test that S is contained in T
 BOOLEAN kTest_TS(kStrategy strat);
 // test LObject
-BOOLEAN kTest_L(LObject* L, ring tailRing,
+BOOLEAN kTest_L(LObject* L, kStrategy strat,
                  BOOLEAN testp = FALSE, int lpos = -1,
                  TSet T = NULL, int tlength = -1);
 // test TObject
-BOOLEAN kTest_T(TObject* T, ring tailRing = NULL, int tpos = -1, char TN = '?');
+BOOLEAN kTest_T(TObject* T, kStrategy strat, int tpos = -1, char TN = '?');
 // test set strat->SevS
 BOOLEAN kTest_S(kStrategy strat);
 #else
 #define kTest(A)        (TRUE)
 #define kTest_TS(A)     (TRUE)
-#define kTest_T(T)      (TRUE)
+#define kTest_T(T,S)    (TRUE)
 #define kTest_S(T)      (TRUE)
 #define kTest_L(T,R)    (TRUE)
 #endif
@@ -702,12 +688,14 @@ void f5c (kStrategy strat, int& olddeg, int& minimcnt, int& hilbeledeg,
 //            (strat == NULL)
 //         -1 tailRing change could not be performed due to exceeding exp
 //            bound of currRing
+//  reduce should be set inside "kNF" (only relevant for rings)
 int ksReducePoly(LObject* PR,
                  TObject* PW,
                  poly spNoether = NULL,
                  number *coef = NULL,
                  poly *mon =NULL,
-                 kStrategy strat = NULL);
+                 kStrategy strat = NULL,
+                 BOOLEAN redtail = FALSE);
 
 /* like ksReducePoly, but if the reducer has only 1 term we still
  * compute a possible coefficient multiplier for PR. this comes from
@@ -768,7 +756,7 @@ int ksReducePolySigRing(LObject* PR,
 
 // Reduces PR at Current->next with PW
 // Assumes PR != NULL, Current contained in PR
-//         Current->next != NULL, LM(PW) devides LM(Current->next)
+//         Current->next != NULL, LM(PW) divides LM(Current->next)
 // Changes: PR
 // Const:   PW
 // Return: see ksReducePoly
@@ -813,7 +801,6 @@ KINLINE void ksOldSpolyTail(poly p1, poly q, poly q2, poly spNoether, ring r = c
 //             exponent bound of strat->tailRing
 //      FALSE, otherwise
 BOOLEAN kCheckSpolyCreation(LObject* L, kStrategy strat, poly &m1, poly &m2);
-#ifdef HAVE_RINGS
 // return TRUE if gcdpoly creation of R[atR] and S[atS] does not violate
 //             exponent bound of strat->tailRing
 //      FALSE, otherwise
@@ -822,7 +809,6 @@ poly preIntegerCheck(ideal F, ideal Q);
 void postReduceByMon(LObject* h, kStrategy strat);
 void postReduceByMonSig(LObject* h, kStrategy strat);
 void finalReduceByMon(kStrategy strat);
-#endif
 // change strat->tailRing and adjust all data in strat, L, and T:
 // new tailRing has larger exponent bound
 // do nothing and return FALSE if exponent bound increase would result in
@@ -850,15 +836,19 @@ KINLINE void clearS (poly p, unsigned long p_sev, int* at, int* k,
 /* shiftgb stuff */
 #include "kernel/GBEngine/shiftgb.h"
 
-poly pMove2CurrTail(poly p, kStrategy strat);
-
-poly pMoveCurrTail2poly(poly p, kStrategy strat);
-
-poly pCopyL2p(LObject h, kStrategy strat);
+#ifdef HAVE_SHIFTBBA
+static inline int kFindInL1(const poly p, const kStrategy strat)
+{
+  for(int i=strat->Ll;i>=0;i--)
+  {
+    if (p==strat->L[i].p1) return i;
+  }
+  return -1;
+}
 
 void enterTShift(LObject p, kStrategy strat, int atT = -1);
 
-void enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS);
+BOOLEAN enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS);
 
 void enterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR);
 
@@ -869,6 +859,8 @@ poly redtailBbaShift (LObject* L, int pos, kStrategy strat, BOOLEAN withT, BOOLE
 int redFirstShift (LObject* h,kStrategy strat); // ok
 
 ideal bbaShift(ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat);
+#endif
+
 // test syz strategy: // will be removed soon
 EXTERN_VAR int (*test_PosInT)(const TSet T,const int tl,LObject &h);
 EXTERN_VAR int (*test_PosInL)(const LSet set, const int length,
@@ -878,11 +870,9 @@ static inline void kDeleteLcm(LObject *P)
 {
  if (P->lcm!=NULL)
  {
- #ifdef HAVE_RINGS
    if (rField_is_Ring(currRing))
      pLmDelete(P->lcm);
    else
- #endif
      pLmFree(P->lcm);
    P->lcm=NULL;
  }
