@@ -46,7 +46,7 @@ typedef poly (*pShallowCopyDeleteProc)(poly s_p, ring source_r, ring dest_r,
 
 // ro_typ describes what to store at the corresping "data" place in p->exp
 // none of the directly corresponds to a ring ordering (ringorder_*)
-// as each ringorder_* blocks corrsponds to 0..2 sro-blocks
+// as each ringorder_* blocks corresponds to 0..2 sro-blocks
 typedef enum
 {
   ro_dp, // total degree with weights 1
@@ -76,10 +76,11 @@ typedef enum rRingOrder_t
   ringorder_s, ///< s?
   ringorder_lp,
   ringorder_dp,
-  ringorder_rp,
+  ringorder_ip,
   ringorder_Dp,
   ringorder_wp,
   ringorder_Wp,
+  ringorder_Ip, /// degree, ip
   ringorder_ls,
   ringorder_ds,
   ringorder_Ds,
@@ -89,11 +90,15 @@ typedef enum rRingOrder_t
   ringorder_L,
   // the following are only used internally
   ringorder_aa, ///< for idElimination, like a, except pFDeg, pWeigths ignore it
-  ringorder_rs, ///< opposite of ls
+  ringorder_is, ///< opposite of ls
   ringorder_IS, ///< Induced (Schreyer) ordering
   ringorder_unspec
 } rRingOrder_t;
 
+// compat entries:
+#define ringorder_rp ringorder_ip
+#define ringorder_rs ringorder_is
+
 typedef enum rOrderType_t
 {
   rOrderType_General = 0, ///< non-simple ordering as specified by currRing
@@ -190,7 +195,7 @@ typedef struct sro_syz sro_syz;
 
 // due to prefix/suffix nature we need some placeholder:
 // prefix stores here initial state
-// suffix cleares this up
+// suffix clears this up
 struct sro_ISTemp
 {
   short start; // 1st member SHOULD be short "place"
@@ -198,8 +203,8 @@ struct sro_ISTemp
   int*  pVarOffset; // copy!
 };
 
-// So this is the actuall thing!
-// suffix uses last sro_ISTemp (cleares it up afterwards) and
+// So this is the actual thing!
+// suffix uses last sro_ISTemp (clears it up afterwards) and
 // creates this block
 struct sro_IS
 {
@@ -265,7 +270,7 @@ struct ip_sring
   sro_ord*   typ;   /* array of orderings + sizes, OrdSize entries */
   /* if NegWeightL_Size > 0, then NegWeightL_Offset[0..size_1] is index of longs
   in ExpVector whose values need an offset due to negative weights */
-  /* array of NegWeigtL_Size indicies */
+  /* array of NegWeigtL_Size indices */
   int*      NegWeightL_Offset;
 
   int*     VarOffset;
@@ -284,7 +289,7 @@ struct ip_sring
 
   omBin    PolyBin; /* Bin from where monoms are allocated */
   intvec * pModW;   /* std: module weights */
-  poly      ppNoether; /*  variables, set by procedures from hecke/kstd1:
+  poly     ppNoether; /*  variables, set by procedures from hecke/kstd1:
                             the highest monomial below pHEdge */
   void * ext_ref;   /* libsing GAP object */
 // #ifdef HAVE_RINGS
@@ -346,8 +351,10 @@ struct ip_sring
      VarL_Offset[i] gets i-th long var in exp vector */
   int*      VarL_Offset;
 
-  /* mask for getting single exponents */
+  /* mask for getting single exponents, also maxExp */
   unsigned long bitmask;
+  /* wanted maxExp */
+  unsigned long wanted_maxExp;
   /* mask used for divisiblity tests */
   unsigned long divmask; // rComplete
 
@@ -409,7 +416,7 @@ static inline BOOLEAN rIsPluralRing(const ring r)
 static inline BOOLEAN rIsLPRing(const ring r)
 {
   assume(r != NULL);
-#if defined(HAVE_PLURAL) && defined(HAVE_SHIFTBBA)
+#ifdef HAVE_SHIFTBBA
   return (r->isLPring!=0);
 #else
   return FALSE;
@@ -418,11 +425,8 @@ static inline BOOLEAN rIsLPRing(const ring r)
 
 static inline BOOLEAN rIsNCRing(const ring r)
 {
-#ifdef HAVE_PLURAL
-  return rIsLPRing(r) || rIsPluralRing(r);
-#else
-  return FALSE;
-#endif
+  assume(r != NULL);
+  return rIsPluralRing(r) || rIsLPRing(r);
 }
 
 static inline BOOLEAN rIsRatGRing(const ring r)
@@ -471,7 +475,6 @@ BOOLEAN rSamePolyRep(ring r1, ring r2);
 
 void   rUnComplete(ring r);
 
-BOOLEAN rRing_is_Homog(const ring r);
 BOOLEAN rRing_has_CompLastBlock(const ring r);
 BOOLEAN rRing_ord_pure_dp(const ring r);
 BOOLEAN rRing_ord_pure_Dp(const ring r);
@@ -484,8 +487,7 @@ static inline BOOLEAN rField_is_Ring_2toM(const ring r)
 static inline BOOLEAN rField_is_Ring_PtoM(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_PtoM(r->cf) ); }
 
-static inline BOOLEAN rField_is_Ring(const ring r)
-{ assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Ring(r->cf); }
+#define rField_is_Ring(R) nCoeff_is_Ring((R)->cf)
 
 static inline BOOLEAN rField_is_Domain(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Domain(r->cf); }
@@ -568,7 +570,7 @@ BOOLEAN rComplete(ring r, int force = 0);
 /// set all properties of a new ring - also called by rComplete
 void p_SetGlobals(const ring r, BOOLEAN complete = TRUE);
 
-static inline int rBlocks(ring r)
+static inline int rBlocks(const ring r)
 {
   assume(r != NULL);
   int i=0;
@@ -736,6 +738,8 @@ ring   rAssure_InducedSchreyerOrdering(const ring r, BOOLEAN complete = TRUE, in
 
 ring   rAssure_dp_S(const ring r);
 ring   rAssure_dp_C(const ring r);
+ring   rAssure_Dp_C(const ring r);
+ring   rAssure_Wp_C(const ring r, intvec *w);
 ring   rAssure_C_dp(const ring r);
 ring   rAssure_c_dp(const ring r);
 
@@ -767,14 +771,14 @@ inline BOOLEAN rHasMixedOrdering(const ring r) { return (r->MixedOrder); }
 // #define rHasLocalOrMixedOrdering(R) ((R)->OrdSgn==-1)
 
 BOOLEAN rOrd_is_Totaldegree_Ordering(const ring r);
+BOOLEAN rOrd_is_dp(const ring r);
+BOOLEAN rOrd_is_ds(const ring r);
+BOOLEAN rOrd_is_Ds(const ring r);
 
 /// return TRUE if p_SetComp requires p_Setm
 BOOLEAN rOrd_SetCompRequiresSetm(const ring r);
 rOrderType_t    rGetOrderType(ring r);
 
-/// returns TRUE if var(i) belongs to p-block
-BOOLEAN rIsPolyVar(int i, const ring r);
-
 static inline BOOLEAN rOrd_is_Comp_dp(const ring r)
 {
   assume(r != NULL);
@@ -797,7 +801,6 @@ ring rModifyRing(ring r, BOOLEAN omit_degree,
 
 /// construct Wp, C ring
 ring rModifyRing_Wp(ring r, int* weights);
-void rModify_a_to_A(ring r);
 
 void rKillModifiedRing(ring r);
 // also frees weights
@@ -815,7 +818,7 @@ int64 * rGetWeightVec(const ring r);
 void rSetWeightVec(ring r, int64 *wv);
 
 /////////////////////////////
-// Auxillary functions
+// Auxiliary functions
 //
 
 /* return the varIndex-th ring variable as a poly;
@@ -826,7 +829,6 @@ BOOLEAN rSetISReference(const ring r, const ideal F, const int i = 0, const int
 
 /// return the position of the p^th IS block order block in r->typ[]...
 int rGetISPos(const int p, const ring r);
-void pISUpdateComponents(ideal F, const intvec *const V, const int MIN, const ring r);
 
 BOOLEAN rCheckIV(const intvec *iv);
 int rTypeOfMatrixOrder(const intvec *order);
@@ -841,4 +843,7 @@ ring rPlusVar(const ring r, char *v,int left);
 
 /// undo rPlusVar
 ring rMinusVar(const ring r, char *v);
+
+static inline ring rIncRefCnt(ring r) { r->ref++; return r; }
+static inline void rDecRefCnt(ring r) { r->ref--; }
 #endif
